@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2022, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,6 +18,8 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
+ * SPDX-License-Identifier: curl
+ *
  ***************************************************************************/
 #include "tool_setup.h"
 
@@ -33,7 +35,7 @@
 #include <fcntl.h>
 #endif
 
-#include <curl/mprintf.h>
+#include <curlx.h>
 
 #include "tool_findfile.h"
 
@@ -47,11 +49,11 @@ struct finder {
 
 /* The order of the variables below is important, as the index number is used
    in the findfile() function */
-static const struct finder list[] = {
+static const struct finder conf_list[] = {
   { "CURL_HOME", NULL, FALSE },
-  { "XDG_CONFIG_HOME", NULL, FALSE }, /* index == 1, used in the code */
+  { "XDG_CONFIG_HOME", NULL, TRUE },
   { "HOME", NULL, FALSE },
-#ifdef WIN32
+#ifdef _WIN32
   { "USERPROFILE", NULL, FALSE },
   { "APPDATA", NULL, FALSE },
   { "USERPROFILE", "\\Application Data", FALSE},
@@ -70,9 +72,9 @@ static char *checkhome(const char *home, const char *fname, bool dotscore)
   for(i = 0; i < (dotscore ? 2 : 1); i++) {
     char *c;
     if(dotscore)
-      c = curl_maprintf("%s" DIR_CHAR "%c%s", home, pref[i], &fname[1]);
+      c = aprintf("%s" DIR_CHAR "%c%s", home, pref[i], &fname[1]);
     else
-      c = curl_maprintf("%s" DIR_CHAR "%s", home, fname);
+      c = aprintf("%s" DIR_CHAR "%s", home, fname);
     if(c) {
       int fd = open(c, O_RDONLY);
       if(fd >= 0) {
@@ -95,38 +97,35 @@ static char *checkhome(const char *home, const char *fname, bool dotscore)
  *
  * 1. Iterate over the environment variables in order, and if set, check for
  *    the given file to be accessed there, then it is a match.
- * 2. Non-windows: try getpwuid
+ * 2. Non-Windows: try getpwuid
  */
 char *findfile(const char *fname, int dotscore)
 {
   int i;
-  bool xdg = FALSE;
   DEBUGASSERT(fname && fname[0]);
   DEBUGASSERT((dotscore != 1) || (fname[0] == '.'));
 
   if(!fname[0])
     return NULL;
 
-  for(i = 0; list[i].env; i++) {
-    char *home = curl_getenv(list[i].env);
+  for(i = 0; conf_list[i].env; i++) {
+    char *home = curl_getenv(conf_list[i].env);
     if(home) {
       char *path;
       const char *filename = fname;
-      if(i == 1 /* XDG_CONFIG_HOME */)
-        xdg = TRUE;
       if(!home[0]) {
         curl_free(home);
         continue;
       }
-      if(list[i].append) {
-        char *c = curl_maprintf("%s%s", home, list[i].append);
+      if(conf_list[i].append) {
+        char *c = aprintf("%s%s", home, conf_list[i].append);
         curl_free(home);
         if(!c)
           return NULL;
         home = c;
       }
-      if(list[i].withoutdot) {
-        if(!dotscore || xdg) {
+      if(conf_list[i].withoutdot) {
+        if(!dotscore) {
           /* this is not looking for .curlrc, or the XDG_CONFIG_HOME was
              defined so we skip the extended check */
           curl_free(home);
