@@ -27,16 +27,15 @@
 import logging
 import os
 import re
+import shutil
 import socket
 import subprocess
-import sys
+import tempfile
 from configparser import ConfigParser, ExtendedInterpolation
 from datetime import timedelta
 from typing import Optional
 
-import pytest
-
-from .certs import CertificateSpec, TestCA, Credentials
+from .certs import CertificateSpec, Credentials, TestCA
 from .ports import alloc_ports
 
 
@@ -52,10 +51,9 @@ def init_config_from(conf_path):
 
 
 TESTS_HTTPD_PATH = os.path.dirname(os.path.dirname(__file__))
-DEF_CONFIG = init_config_from(os.path.join(TESTS_HTTPD_PATH, 'config.ini'))
-
-TOP_PATH = os.path.dirname(os.path.dirname(TESTS_HTTPD_PATH))
-CURL = os.path.join(TOP_PATH, 'src/curl')
+TOP_PATH = os.path.join(os.getcwd(), os.path.pardir)
+DEF_CONFIG = init_config_from(os.path.join(TOP_PATH, 'tests', 'http', 'config.ini'))
+CURL = os.path.join(TOP_PATH, 'src', 'curl')
 
 
 class EnvConfig:
@@ -64,55 +62,63 @@ class EnvConfig:
         self.tests_dir = TESTS_HTTPD_PATH
         self.gen_dir = os.path.join(self.tests_dir, 'gen')
         self.project_dir = os.path.dirname(os.path.dirname(self.tests_dir))
+        self.build_dir = TOP_PATH
         self.config = DEF_CONFIG
         # check cur and its features
         self.curl = CURL
         if 'CURL' in os.environ:
             self.curl = os.environ['CURL']
         self.curl_props = {
-            'version': None,
-            'os': None,
-            'fullname': None,
-            'features': [],
-            'protocols': [],
-            'libs': [],
-            'lib_versions': [],
+            'version_string': '',
+            'version': '',
+            'os': '',
+            'fullname': '',
+            'features_string': '',
+            'features': set(),
+            'protocols_string': '',
+            'protocols': set(),
+            'libs': set(),
+            'lib_versions': set(),
         }
         self.curl_is_debug = False
         self.curl_protos = []
         p = subprocess.run(args=[self.curl, '-V'],
                            capture_output=True, text=True)
         if p.returncode != 0:
-            assert False, f'{self.curl} -V failed with exit code: {p.returncode}'
+            raise RuntimeError(f'{self.curl} -V failed with exit code: {p.returncode}')
         if p.stderr.startswith('WARNING:'):
             self.curl_is_debug = True
-        for l in p.stdout.splitlines(keepends=False):
-            if l.startswith('curl '):
-                m = re.match(r'^curl (?P<version>\S+) (?P<os>\S+) (?P<libs>.*)$', l)
+        for line in p.stdout.splitlines(keepends=False):
+            if line.startswith('curl '):
+                self.curl_props['version_string'] = line
+                m = re.match(r'^curl (?P<version>\S+) (?P<os>\S+) (?P<libs>.*)$', line)
                 if m:
                     self.curl_props['fullname'] = m.group(0)
                     self.curl_props['version'] = m.group('version')
                     self.curl_props['os'] = m.group('os')
-                    self.curl_props['lib_versions'] = [
+                    self.curl_props['lib_versions'] = {
                         lib.lower() for lib in m.group('libs').split(' ')
-                    ]
-                    self.curl_props['libs'] = [
-                        re.sub(r'/.*', '', lib) for lib in self.curl_props['lib_versions']
-                    ]
-            if l.startswith('Features: '):
-                self.curl_props['features'] = [
-                    feat.lower() for feat in l[10:].split(' ')
-                ]
-            if l.startswith('Protocols: '):
-                self.curl_props['protocols'] = [
-                    prot.lower() for prot in l[11:].split(' ')
-                ]
+                    }
+                    self.curl_props['libs'] = {
+                        re.sub(r'/[a-z0-9.-]*', '', lib) for lib in self.curl_props['lib_versions']
+                    }
+            if line.startswith('Features: '):
+                self.curl_props['features_string'] = line[10:]
+                self.curl_props['features'] = {
+                    feat.lower() for feat in line[10:].split(' ')
+                }
+            if line.startswith('Protocols: '):
+                self.curl_props['protocols_string'] = line[11:]
+                self.curl_props['protocols'] = {
+                    prot.lower() for prot in line[11:].split(' ')
+                }
 
         self.ports = alloc_ports(port_specs={
             'ftp': socket.SOCK_STREAM,
             'ftps': socket.SOCK_STREAM,
             'http': socket.SOCK_STREAM,
             'https': socket.SOCK_STREAM,
+            'nghttpx_https': socket.SOCK_STREAM,
             'proxy': socket.SOCK_STREAM,
             'proxys': socket.SOCK_STREAM,
             'h2proxys': socket.SOCK_STREAM,
@@ -138,11 +144,14 @@ class EnvConfig:
         self.domain2 = f"two.{self.tld}"
         self.ftp_domain = f"ftp.{self.tld}"
         self.proxy_domain = f"proxy.{self.tld}"
+        self.expired_domain = f"expired.{self.tld}"
         self.cert_specs = [
             CertificateSpec(domains=[self.domain1, self.domain1brotli, 'localhost', '127.0.0.1'], key_type='rsa2048'),
             CertificateSpec(domains=[self.domain2], key_type='rsa2048'),
             CertificateSpec(domains=[self.ftp_domain], key_type='rsa2048'),
             CertificateSpec(domains=[self.proxy_domain, '127.0.0.1'], key_type='rsa2048'),
+            CertificateSpec(domains=[self.expired_domain], key_type='rsa2048',
+                            valid_from=timedelta(days=-100), valid_to=timedelta(days=-10)),
             CertificateSpec(name="clientsX", sub_specs=[
                CertificateSpec(name="user1", client=True),
             ]),
@@ -179,20 +188,31 @@ class EnvConfig:
                 if m:
                     self._caddy_version = m.group(1)
                 else:
-                    raise f'Unable to determine cadd version from: {p.stdout}'
-            except:
+                    raise RuntimeError(f'Unable to determine cadd version from: {p.stdout}')
+            # TODO: specify specific exceptions here
+            except:  # noqa: E722
                 self.caddy = None
 
         self.vsftpd = self.config['vsftpd']['vsftpd']
         self._vsftpd_version = None
         if self.vsftpd is not None:
             try:
-                p = subprocess.run(args=[self.vsftpd, '-v'],
-                                   capture_output=True, text=True)
-                if p.returncode != 0:
-                    # not a working vsftpd
-                    self.vsftpd = None
-                m = re.match(r'vsftpd: version (\d+\.\d+\.\d+)', p.stderr)
+                with tempfile.TemporaryFile('w+') as tmp:
+                    p = subprocess.run(args=[self.vsftpd, '-v'],
+                                       capture_output=True, text=True, stdin=tmp)
+                    if p.returncode != 0:
+                        # not a working vsftpd
+                        self.vsftpd = None
+                    if p.stderr:
+                        ver_text = p.stderr
+                    else:
+                        # Oddly, some versions of vsftpd write to stdin (!)
+                        # instead of stderr, which is odd but works. If there
+                        # is nothing on stderr, read the file on stdin and use
+                        # any data there instead.
+                        tmp.seek(0)
+                        ver_text = tmp.read()
+                m = re.match(r'vsftpd: version (\d+\.\d+\.\d+)', ver_text)
                 if m:
                     self._vsftpd_version = m.group(1)
                 elif len(p.stderr) == 0:
@@ -200,9 +220,11 @@ class EnvConfig:
                     self._vsftpd_version = 'unknown'
                 else:
                     raise Exception(f'Unable to determine VsFTPD version from: {p.stderr}')
-            except Exception as e:
+            except Exception:
                 self.vsftpd = None
 
+        self._tcpdump = shutil.which('tcpdump')
+
     @property
     def httpd_version(self):
         if self._httpd_version is None and self.apxs is not None:
@@ -213,8 +235,8 @@ class EnvConfig:
                     log.error(f'{self.apxs} failed to query HTTPD_VERSION: {p}')
                 else:
                     self._httpd_version = p.stdout.strip()
-            except Exception as e:
-                log.error(f'{self.apxs} failed to run: {e}')
+            except Exception:
+                log.exception(f'{self.apxs} failed to run')
         return self._httpd_version
 
     def versiontuple(self, v):
@@ -241,13 +263,13 @@ class EnvConfig:
 
     def get_incomplete_reason(self) -> Optional[str]:
         if self.httpd is None or len(self.httpd.strip()) == 0:
-            return f'httpd not configured, see `--with-test-httpd=<path>`'
+            return 'httpd not configured, see `--with-test-httpd=<path>`'
         if not os.path.isfile(self.httpd):
             return f'httpd ({self.httpd}) not found'
         if not os.path.isfile(self.apachectl):
             return f'apachectl ({self.apachectl}) not found'
         if self.apxs is None:
-            return f"command apxs not found (commonly provided in apache2-dev)"
+            return "command apxs not found (commonly provided in apache2-dev)"
         if not os.path.isfile(self.apxs):
             return f"apxs ({self.apxs}) not found"
         return None
@@ -264,6 +286,10 @@ class EnvConfig:
     def vsftpd_version(self):
         return self._vsftpd_version
 
+    @property
+    def tcpdmp(self) -> Optional[str]:
+        return self._tcpdump
+
 
 class Env:
 
@@ -287,7 +313,7 @@ class Env:
 
     @staticmethod
     def have_ssl_curl() -> bool:
-        return 'ssl' in Env.CONFIG.curl_props['features']
+        return Env.curl_has_feature('ssl') or Env.curl_has_feature('multissl')
 
     @staticmethod
     def have_h2_curl() -> bool:
@@ -307,10 +333,22 @@ class Env:
             return not Env.curl_uses_lib('ngtcp2') and Env.curl_uses_lib('nghttp3')
         return False
 
+    @staticmethod
+    def curl_version_string() -> str:
+        return Env.CONFIG.curl_props['version_string']
+
+    @staticmethod
+    def curl_features_string() -> str:
+        return Env.CONFIG.curl_props['features_string']
+
     @staticmethod
     def curl_has_feature(feature: str) -> bool:
         return feature.lower() in Env.CONFIG.curl_props['features']
 
+    @staticmethod
+    def curl_protocols_string() -> str:
+        return Env.CONFIG.curl_props['protocols_string']
+
     @staticmethod
     def curl_has_protocol(protocol: str) -> bool:
         return protocol.lower() in Env.CONFIG.curl_props['protocols']
@@ -324,7 +362,7 @@ class Env:
         return 'unknown'
 
     @staticmethod
-    def curl_lib_version_at_least(libname: str, min_version) -> str:
+    def curl_lib_version_at_least(libname: str, min_version) -> bool:
         lversion = Env.curl_lib_version(libname)
         if lversion != 'unknown':
             return Env.CONFIG.versiontuple(min_version) <= \
@@ -383,6 +421,10 @@ class Env:
     def vsftpd_version() -> str:
         return Env.CONFIG.vsftpd_version
 
+    @staticmethod
+    def tcpdump() -> Optional[str]:
+        return Env.CONFIG.tcpdmp
+
     def __init__(self, pytestconfig=None):
         self._verbose = pytestconfig.option.verbose \
             if pytestconfig is not None else 0
@@ -428,6 +470,10 @@ class Env:
     def project_dir(self) -> str:
         return self.CONFIG.project_dir
 
+    @property
+    def build_dir(self) -> str:
+        return self.CONFIG.build_dir
+
     @property
     def ca(self):
         return self._ca
@@ -436,6 +482,10 @@ class Env:
     def htdocs_dir(self) -> str:
         return self.CONFIG.htdocs_dir
 
+    @property
+    def tld(self) -> str:
+        return self.CONFIG.tld
+
     @property
     def domain1(self) -> str:
         return self.CONFIG.domain1
@@ -456,6 +506,10 @@ class Env:
     def proxy_domain(self) -> str:
         return self.CONFIG.proxy_domain
 
+    @property
+    def expired_domain(self) -> str:
+        return self.CONFIG.expired_domain
+
     @property
     def http_port(self) -> int:
         return self.CONFIG.ports['http']
@@ -464,6 +518,10 @@ class Env:
     def https_port(self) -> int:
         return self.CONFIG.ports['https']
 
+    @property
+    def nghttpx_https_port(self) -> int:
+        return self.CONFIG.ports['nghttpx_https']
+
     @property
     def h3_port(self) -> int:
         return self.https_port
@@ -541,33 +599,30 @@ class Env:
     def ci_run(self) -> bool:
         return "CURL_CI" in os.environ
 
-    def authority_for(self, domain: str, alpn_proto: Optional[str] = None):
+    def port_for(self, alpn_proto: Optional[str] = None):
         if alpn_proto is None or \
                 alpn_proto in ['h2', 'http/1.1', 'http/1.0', 'http/0.9']:
-            return f'{domain}:{self.https_port}'
+            return self.https_port
         if alpn_proto in ['h3']:
-            return f'{domain}:{self.h3_port}'
-        return f'{domain}:{self.http_port}'
+            return self.h3_port
+        return self.http_port
 
-    def make_data_file(self, indir: str, fname: str, fsize: int) -> str:
+    def authority_for(self, domain: str, alpn_proto: Optional[str] = None):
+        return f'{domain}:{self.port_for(alpn_proto=alpn_proto)}'
+
+    def make_data_file(self, indir: str, fname: str, fsize: int,
+                       line_length: int = 1024) -> str:
+        if line_length < 11:
+            raise RuntimeError('line_length less than 11 not supported')
         fpath = os.path.join(indir, fname)
         s10 = "0123456789"
-        s = (101 * s10) + s10[0:3]
+        s = round((line_length / 10) + 1) * s10
+        s = s[0:line_length-11]
         with open(fpath, 'w') as fd:
-            for i in range(int(fsize / 1024)):
+            for i in range(int(fsize / line_length)):
                 fd.write(f"{i:09d}-{s}\n")
-            remain = int(fsize % 1024)
+            remain = int(fsize % line_length)
             if remain != 0:
-                i = int(fsize / 1024) + 1
-                s = f"{i:09d}-{s}\n"
-                fd.write(s[0:remain])
+                i = int(fsize / line_length) + 1
+                fd.write(f"{i:09d}-{s}"[0:remain-1] + "\n")
         return fpath
-
-    def make_clients(self):
-        client_dir = os.path.join(self.project_dir, 'tests/http/clients')
-        p = subprocess.run(['make'], capture_output=True, text=True,
-                           cwd=client_dir)
-        if p.returncode != 0:
-            pytest.exit(f"`make`in {client_dir} failed:\n{p.stderr}")
-            return False
-        return True
